/* Mathematik C fuer Informatikstudierende -- Serie 7 -- 15.12.2013
   Anna Schleimer, Thure Duehrsen
   Aufgabe 7.3: Vektoriteration
*/


#include <stdlib.h>
#include <stdio.h>
/* #include <strings.h> */
#include "Tuple.h"
#include "Vector.h"
#include "Matrix.h"

MatrixOfDouble MatrixOfDouble_Create  (unsigned int rows,
                                       unsigned int cols,
				       MatrixOrdering ordering) {

  /* Bekommt zwei natuerliche Zahlen rows und cols, die fuer die
     Anzahl der Zeilen bzw. der Spalten stehen, und fordert
     ausreichend Speicher fuer m * n Double-Werte an. Alle Eintraege
     werden mit -1 vorbesetzt.
  */

  MatrixOfDouble result;

  result.numberOfRows = rows;
  result.numberOfColumns = cols;
  result.ordering = ordering;
  
  if (rows <= 0) {

    fprintf (stderr, "Invalid matrix initialiser:"
             " Number of rows is not positive\n");
    result.values = NULL;
    return result;

  }

  if (cols <= 0) {

    fprintf (stderr, "Invalid matrix initialiser:"
             " Number of columns is not positive\n");
    result.values = NULL;
    return result;

  }

  result.values = TupleOfDouble_Create (rows*cols);

  if (result.values == NULL) {

    fprintf (stderr, "Unable to create matrix: Nonexistent entries\n");
    
  }

  return result;
  

}

void             MatrixOfDouble_CopyValues  (MatrixOfDouble dest,
					     MatrixOfDouble src) {

  /* Performs dest = src, component-wise. */

  unsigned int i, j;

  if (src.values == NULL) {

    fprintf (stderr, "MatrixOfDouble_CopyValues: Matrix to copy is NULL\n");
    exit (1);

  }

  if (dest.values == NULL) {

    fprintf (stderr, "MatrixOfDouble_CopyValues:"
	     " Matrix to hold result is NULL\n");
    exit (1);

  }

  if (src.ordering != dest.ordering) {

    fprintf (stderr, "MatrixOfDouble_CopyValues:"
	     " Matrices do not have the same ordering\n");
    exit (1);

  }

  if (src.numberOfRows != dest.numberOfRows) {

    fprintf (stderr, "MatrixOfDouble_CopyValues:"
	     " Matrices do not have the same number of rows\n");
    exit (1);

  }

  if (src.numberOfColumns != dest.numberOfColumns) {

    fprintf (stderr, "MatrixOfDouble_CopyValues:"
	     " Matrices do not have the same number of columns\n");
    exit (1);

  }

  for (i = 1; i <= dest.numberOfRows; i++) {

    for (j = 1; j <= dest.numberOfColumns; j++) {

      MatrixOfDouble_SetEntry_1
	(dest, i, j, MatrixOfDouble_GetEntry_1(src, i, j));

    }
  
  }

}

MatrixOfDouble   MatrixOfDouble_Copy (MatrixOfDouble src) {

  /* Performs dest = src, component-wise. */

  MatrixOfDouble dest;
  dest = MatrixOfDouble_Create (src.numberOfRows,
				src.numberOfColumns,
				src.ordering);

  if (src.values == NULL) {

    fprintf (stderr, "VectorOfDouble_Copy: Vector to copy is NULL\n");
    exit (1);

  }

  MatrixOfDouble_CopyValues (dest, src);

  return dest;

}

void           MatrixOfDouble_Destroy (MatrixOfDouble* mp) {

  mp -> numberOfRows    = 0;
  mp -> numberOfColumns = 0;

  free (mp -> values);

  /* ordering bleibt unveraendert */

  

}

MatrixOfDouble   MatrixOfDouble_LoadFromFile (char* filename) {

  FILE* F = NULL;

  MatrixOfDouble result;

  unsigned int numberOfRows = -1;
  unsigned int numberOfColumns = -1;
  /* halten die Anzahl der Zeilen und Spalten fest, um die Matrix
     erzeugen zu koennen */

  unsigned int i, j; /* Schleifenzaehler */

  double entry; /* bewahrt einen aus der Datei eingelesenen
		   Matrixeintrag auf, damit er leicht in die Matrix
		   eingebaut werden kann */

  int fscanf_return_value = -42; /* bewahrt den Rueckgabewert von fscanf zur
				    Auswertung auf*/
  
  unsigned int z = 0 ; /* zaehlt die Anzahl der erfolgreich eingelesenen
			  Matrixeintraege */

  MatrixOrdering ordering = ROWWISE; /* diese Funktion bastelt immer
					eine zeilenweise organisierte
					Matrix */
  
  F = fopen (filename, "r");

  if (F == NULL) {

    fprintf (stderr, "MatrixOfDouble_LoadFromFile"
	     ": Unable to read file %s\n", filename);
    fclose (F);
    exit (1);

  }

  fscanf (F, "%u", &numberOfRows);
  /* hier fehlt Fehlerbehandlung */

  fscanf (F, "%u", &numberOfColumns);
  /* hier fehlt Fehlerbehandlung */

  result = MatrixOfDouble_Create (numberOfRows, numberOfColumns, ordering);

  for (i = 1; i <= numberOfRows; i++) {

    for (j = 1; j <= numberOfColumns; j++) {

      fscanf_return_value = fscanf (F, "%lf", &entry);

      z = z + fscanf_return_value;

      MatrixOfDouble_SetEntry_1 (result, i, j, entry);

    }

  }

  for (i = 0; i < numberOfRows * numberOfColumns; i++) {

    fscanf (F, "%lf", &result.values[i]);
  }
  
  fclose (F);

  if (z < numberOfRows * numberOfColumns) {

    fprintf (stderr, "MatrixOfDouble_LoadFromFile"
  	     ": Es konnten nur %u der %u Eintraege"
  	     " der Matrix gelesen werden!\n",
  	     z, numberOfRows * numberOfColumns);
    fclose (F);
    exit (1);

  }
  
  return result;
  

}

void           MatrixOfDouble_Print (MatrixOfDouble A) {
  /* aus v03 */

  /* TODO
     Zahlenformat flexibel gestalten
   */
  
  unsigned int i; /* Iteriert ueber die Zeilen */
  unsigned int j; /* Iteriert ueber die Spalten */

  if (A.values == NULL) {

    fprintf (stderr, "MatrixOfDouble_Print: Nonexistent matrix\n");
    exit (1);

  }

  for (i = 1; i <= A.numberOfRows; i++) {

    printf ("[");

    for (j = 1; j < A.numberOfColumns; j++) {

      printf ("% 3.8f,  ", MatrixOfDouble_GetEntry_1 (A, i,j));

    }

    printf ("% 3.8f", MatrixOfDouble_GetEntry_1 (A, i, A.numberOfColumns));

    printf ("]\n");

  }

}

MatrixOrdering   MatrixOfDouble_getOrdering (MatrixOfDouble A) {

  if (A.values == NULL) {

    fprintf (stderr, "MatrixOfDouble_GetOrdering: Nonexistent matrix\n");
    exit (1);

  }

  return A.ordering;

}

void             MatrixOfDouble_setOrdering (MatrixOfDouble A,
					     MatrixOrdering o) {

  if (A.values == NULL) {

    fprintf (stderr, "MatrixOfDouble_SetOrdering: Nonexistent matrix\n");
    exit (1);

  }

  /* hier muss evtl. noch mehr Fehlerbehandlung hin.

     Aber das hier kann man auch zum Transponieren von Matrizen benutzen!!!!
  */ 

  A.ordering = o;

}

double           MatrixOfDouble_GetEntry_0 (MatrixOfDouble A,
					    unsigned int i,
					    unsigned int j) {

  unsigned long int longIndex = 0;

  if (i+1 > A.numberOfRows) {

    fprintf (stderr, "MatrixOfDouble_GetEntry: Row index out of bounds\n");
    exit (1);

  }

  if (j+1 > A.numberOfColumns) {

    fprintf (stderr, "MatrixOfDouble_GetEntry: Column index out of bounds\n");
    exit (1);

  }

  if (A.values == NULL) {

    fprintf (stderr, "MatrixOfDouble_GetEntry: Nonexistent matrix\n");
    exit (1);

  }

  longIndex = i * A.numberOfColumns + j;

  return A.values[longIndex];
  
}

double           MatrixOfDouble_GetEntry_1 (MatrixOfDouble A,
					    unsigned int i,
					    unsigned int j) {

  return MatrixOfDouble_GetEntry_0 (A, i-1, j-1);

}

void             MatrixOfDouble_SetEntry_1 (MatrixOfDouble A,
					    unsigned int i,
					    unsigned int j,
					    double x) {

  unsigned long int longIndex = 0;

  if (i == 0 || i > A.numberOfRows) {

    fprintf (stderr, "MatrixOfDouble_SetEntry: Row index out of bounds\n");
    exit (1);

  }

  if (j == 0 || j > A.numberOfColumns) {

    fprintf (stderr, "MatrixOfDouble_SetEntry: Column index out of bounds\n");
    exit (1);

  }

  if (A.values == NULL) {

    fprintf (stderr, "MatrixOfDouble_SetEntry: Nonexistent matrix\n");
    exit (1);

  }

  longIndex = (i-1) * A.numberOfColumns + (j-1);

  A.values[longIndex] = x;

}

void             MatrixVectorMult_inPlace (VectorOfDouble y,
					   MatrixOfDouble A,
					   VectorOfDouble x) {

  /* Berechnet y = A * x, wobei der Vektor y bereits existiert,
     d.h. nicht mehr mit VectorOfDouble_Create (...) angelegt werden
     muss.
  */

  unsigned int i = 0; /* Iteriert ueber die Eintraege von y */
  unsigned int j = 0; /* Dient zur Berechnung eines Eintrags */

  if (A.values == NULL) {

    fprintf (stderr, "MatrixVectorMult_inPlace: Matrix is NULL\n");
    exit (1);

  }

  if (y.values == NULL) {

    fprintf (stderr, "MatrixVectorMult_inPlace: Result vector is NULL\n");
    exit (1);

  }

  if (x.values == NULL) {

    fprintf (stderr, "MatrixVectorMult_inPlace:"
	     " Vector to be multiplied is NULL\n");
    exit (1);

  }

  if (A.numberOfColumns != x.length) {

    fprintf (stderr, "MatrixVectorMult_inPlace: Incompatible dimensions"
	     " of vector and matrix\n");
    exit (1);

  }

  for (i = 1; i <= x.length; i++) {

    double entry = 0;

    for (j = 1; j <= A.numberOfColumns; j++) {

      entry = entry +
	MatrixOfDouble_GetEntry_1 (A,i,j) * VectorOfDouble_GetEntry_1 (x,j);

    }

    VectorOfDouble_SetEntry_1 (y, i, entry);

  }

}

VectorOfDouble   MatrixVectorMult (MatrixOfDouble A, VectorOfDouble x) {

  VectorOfDouble y;
  
  y = VectorOfDouble_Create (x.length);

  MatrixVectorMult_inPlace (y, A, x);

  return y;

}

void             VectorIteration (MatrixOfDouble A,
				  VectorOfDouble* vp,
				  unsigned int max) {

  unsigned int i;
  VectorOfDouble dummy;

  /* Zuerst die Fehlerbehandlung: ein Aufruf von
     MatrixVectorMult_inPlace nimmt uns viel Arbeit ab! */

  dummy = VectorOfDouble_Create (vp->length);

  MatrixVectorMult_inPlace (dummy, A, *vp);

  VectorOfDouble_Destroy (&dummy);

  /* Wenn die Programmausfuehrung bis hierhin ueberlebt, passen
     die Matrix und der Vektor zueinander. */

  for (i = 1; i <= max; i++) {

    *vp = MatrixVectorMult (A, *vp);

    /* VectorOfDouble_Normalise (*vp); */

    /* if (i >= max - 5) { */

    /*   printf ("s%u = ",i); VectorOfDouble_Print (*vp); printf ("\n"); */

    /* } */

  }

}

void             NormalisedVectorIteration (MatrixOfDouble A,
					    VectorOfDouble* vp,
					    unsigned int max) {

  unsigned int i;
  VectorOfDouble dummy;

  /* Zuerst die Fehlerbehandlung: ein Aufruf von
     MatrixVectorMult_inPlace nimmt uns viel Arbeit ab! */

  dummy = VectorOfDouble_Create (vp->length);

  MatrixVectorMult_inPlace (dummy, A, *vp);

  VectorOfDouble_Destroy (&dummy);

  /* Wenn die Programmausfuehrung bis hierhin ueberlebt, passen
     die Matrix und der Vektor zueinander. */

  for (i = 1; i <= max; i++) {

    *vp = MatrixVectorMult (A, *vp);

    VectorOfDouble_Normalise (*vp);

    /* if (i >= max - 5) { */

    /*   printf ("s%u = ",i); VectorOfDouble_Print (*vp); printf ("\n"); */

    /* } */

  }

}

